/// <reference path="../Scripts/jspack-vsdoc.js" />

var common = MindFusion.Common;
var dv = MindFusion.DataViews;
var ui = MindFusion.Common.UI;
var d = MindFusion.Drawing;

var strip = new ui.ToolStrip(document.getElementById("toolstrip"));
strip.theme = "gray";
strip.scrollable = false;
strip.allowDrag = false;

strip.items.add(new ui.ToolStripItem(ui.ToolStripItemType.Label, "&#9198;"));
strip.items.add(new ui.ToolStripItem(ui.ToolStripItemType.Label, "hour:"));

item = new ui.ToolStripItem(ui.ToolStripItemType.Default);
item.template = "<input type='number' id='hour_input' min='0' value='0'></input> 0-23";
strip.items.add(item);

strip.items.add(new ui.ToolStripItem(ui.ToolStripItemType.Label, "&#9197;"));

strip.items.add(new ui.ToolStripItem(ui.ToolStripItemType.Separator));

item = new ui.ToolStripItem(ui.ToolStripItemType.Default);
item.template = "arriving from: <input type='text' id='city_input' value=''></input>";
strip.items.add(item);

// displaying the search navigation
item = new ui.ToolStripItem(ui.ToolStripItemType.Icon);
item.imageSrc = "../images/prev_train.png";
item.size = common.Unit.pixel(36);
strip.items.add(item);

item = new ui.ToolStripItem(ui.ToolStripItemType.Icon);
item.imageSrc = "../images/next_train.png";
item.size = common.Unit.pixel(36);
strip.items.add(item);

strip.items.add(new ui.ToolStripItem(ui.ToolStripItemType.Separator));

strip.itemClick.addEventListener(function (sender, args)
{
    var index = sender.items.indexOfItem(args.item);   
    
        switch (index)
        {
            case 6: 
			    if(result.length > 0)
			    	goPrev();
                break;
            case 7:
				if(result.length > 0)
                	goNext();
                break;            
            default: return;
        }   
});

strip.render();

// create the grid columns
var columns = [];

var column1 = new dv.GridColumn("scheduled");
column1.dataType = dv.StringType;
column1.caption = "Scheduled";
column1.metaData.set("customType", "time");
columns.push(column1);

var column2 = new dv.GridColumn("train_number");
column2.dataType = dv.StringType;
column2.caption = "Number";
columns.push(column2);

var column3 = new dv.GridColumn("from");
column3.dataType = dv.StringType;
column3.caption = "From";
columns.push(column3);

var column4 = new dv.GridColumn("type");
column4.dataType = dv.ImageType;
column4.caption = "Type";
var imgSrcs = ["../images/express.png", "../images/fast.png", "../images/international.png", "../images/slow.png"];
var imageList;
ui.ImagePicker.loadImageList(imgSrcs).then(response =>
{
    imageList = response;
    column4.metaData.set("customEditor", { "imageList": imageList, "width": common.Unit.pixel(100), "itemSize": common.Unit.pixel(50) });
});
columns.push(column4);

var column5 = new dv.GridColumn("track");
column5.dataType = dv.IntegerType;
column5.caption = "Track";
columns.push(column5);

var column6 = new dv.GridColumn("expected");
column6.dataType = dv.StringType;
column1.metaData.set("customType", "time");
column6.caption = "Expected";
columns.push(column6);



// create the grid control
var grid = new dv.Grid(document.getElementById("grid"));
grid.theme = "gray";
// set the model
grid.model = new dv.ArrayModel(trains, columns, "index");
//stop modificaton actions
grid.allowAppend = false;
grid.allowCellSelect = false;

// render the grid control
grid.render();
//sort by the time
grid.sortedColumn = 0; 

//create the popup
grid.rowRendered.addEventListener(function (sender, args)
{
    var cell = grid.getCellElement(args.row, 3);
	
	var data = grid.model.value(args.row, 3);
	
	if (!data) 
		return;
		
	if (!cell.dataset.tooltip)
	{
		// Create a tooltip for the item.
		// When the item is hovered the tooltip will appear over it with the specified offset.
			var tooltip = new ui.Tooltip(cell, "tra la la");
			tooltip.theme = "gray";			
			tooltip.position = ui.TooltipPosition.Center;
			tooltip.trigger = ui.TooltipTrigger.Hover;
			if(data.length > 14)
				tooltip.text = data.substring(10, data.length - 4) + " train";			
			tooltip.render();
			cell.dataset.tooltip = tooltip.element.id;
	}
});


var searchCity = document.getElementById("city_input");
var result=[];
var resultIndex = -1;

//search for the matching city on Enter
searchCity.addEventListener('keyup', ({key}) => {
    if (key === "Enter")
	{
		result = [];
	
		for(var i = 0; i < trains.length; i++)
		{
			var train = trains[i];			
			
			if(train.from.includes(searchCity.value))
				result.push({row: grid.model.getKeyRow(train.index), column: 2});			
			
		}	
		
		if(result.length > 0)
		{
			resultIndex++;

			//if we have reached to the last result
			if( resultIndex >= result.length)
				resultIndex = 0;
	
			if(grid.model.rowCount > result[resultIndex].row &&
				grid.model.columnCount > result[resultIndex].column)		
			{
				grid.focusCell(result[resultIndex].row, result[resultIndex].column);
				grid.bringIntoView(result[resultIndex].row);	
				
			}
		}		
	}
	else
	{
		
		result = [];
	}
});


function goNext()
{
	
		resultIndex++;

		//if we have reached to the last result
		if( resultIndex >= result.length)
			resultIndex = 0;

		if(grid.model.rowCount > result[resultIndex].row &&
			grid.model.columnCount > result[resultIndex].column)		
		{
			grid.focusCell(result[resultIndex].row, result[resultIndex].column);
			grid.bringIntoView(result[resultIndex].row);			
			
		}
		
}

function goPrev()
{
	
		resultIndex--;

		//if we have reached to the last result
		if( resultIndex <= result.length)
			resultIndex = 0;

		if(grid.model.rowCount > result[resultIndex].row &&
			grid.model.columnCount > result[resultIndex].column)		
		{
			grid.focusCell(result[resultIndex].row, result[resultIndex].column);
			grid.bringIntoView(result[resultIndex].row);			
			
		}	
}

//search by the hour
var searchByHour = document.getElementById("hour_input");

searchByHour.addEventListener('change', (event) => {
  	
	var result = "";
	
		for(var i = 0; i < trains.length; i++)
		{
			var train = trains[i];			
			
			if(train.scheduled.startsWith(searchByHour.value) || 
				train.scheduled.startsWith("0" + searchByHour.value))
				{
						if(result == "")
						   result = train;
						 else if (compareTime(result.scheduled, train.scheduled) == -1)
						 	result = train;			
						
			   }			
		}	
		
		if(result != "" && grid.model.rowCount > result.index)		
		{
				grid.focusCell(grid.model.getKeyRow(result.index), 0);
				grid.bringIntoView(grid.model.getKeyRow(result.index));	
						
					
		}		
	
});


//custom drawing of cells to change font color for trains that
//do not arrive on time
grid.customDrawCell.addEventListener(function (sender, args)
{
    if (args.row === grid.model.rowCount) return;

    if (args.column === 5)
    {
		var scheudled_time = grid.model.value(args.row, 0);
		var expecte_time = grid.model.value(args.row, 5);
		var onTime = compareTime(scheudled_time, expecte_time);
		
        var div = document.createElement("div");
      		
		if(onTime == 1)
			div.style.color = "#ff0000";
		else if (onTime == -1)
			div.style.color = "#006600";
	
			
        div.style.textAlign = "center";
        div.style.fontSize = "larger";
        div.style.borderRadius = "5px";
        div.appendChild(args.defaultContent);
        args.content = div;
    }
});

//returns -1, 0, 1, which depends on whether the train is 
//ahead of time, on time or late
function compareTime(scheduled_time, expected_time) {
    var expectedTimeParts = expected_time.split(":");
	var scheduledTimeParts = scheduled_time.split(":");
	
	var scheduled_date = new Date();
	scheduled_date.setHours(scheduledTimeParts[0]);
	scheduled_date.setMinutes(scheduledTimeParts[1]);
	
	var expected_date = new Date();
	expected_date.setHours(expectedTimeParts[0]);
	expected_date.setMinutes(expectedTimeParts[1]);
	
	//the train is late
	if(expected_date.getTime() > scheduled_date.getTime())	
		return 1;
	
	//the train is coming earlier
	if(expected_date.getTime() < scheduled_date.getTime())	
		return -1;
	
	//the train is on time
    return 0;
}


